// gprod.cpp		ver.0.071
//copyright 1998-2005 Beer[Tsutsumi Hiroyuki]

#include	<Be.h>
#if	B_BEOS_VERSION>=0x0520
#include	<Locale.h>
typedef	const char	Char;
#else
#define	_T(str)	str
typedef	char		Char;
#endif

union ARGB{				//argb
	uint32	c;
	struct	{uchar	a,r,g,b;}u;
};
union BGRA{				//bgra
	uint32	c;
	struct	{uchar	b,g,r,a;}u;
};

typedef	signed short	sint16;

#define	MAX_OBJS	256

#define	SELECTVIEWCOLOR	0x00ffcc44
#define	TOOLVIEWCOLOR	0x00ffcc44

//for input(tablet,mouse,key)
enum{				//flags
	TABLET_XY=		0x0001,		//be:tablet_x,be:tablet_y
	TABLET_TILT_XY=	0x0002,		//be:tablet_tilt_x,be:tablet_tilt_y
	TABLET_PRESSURE=0x0004,		//be:tablet_pressure
	TABLET_ERASER=	0x0008,		//be:tablet_eraser
	BUTTONS=		0x0010,		//buttons
	MODIFIERS=		0x0020,		//modifiers
	WHEN=			0x0100,		//when
	WHERE=			0x0200,		//be:view_where
	TRANSIT=		0x0400,		//be:transit
};

enum{				//mouse
	NONE=0,
	PRESS,
	RELEASE,
	PRESSMOVE,
	RELEASEMOVE,
	CTRLMOVE,
	CANSEL
};

enum{				//object-type
	PICK=0,
	POINT=0x10,
	LINE=0x20,
	CURVE,
	RECT=0x30,
	ELLIPSE=0x40,
	POLY=0x50,
	SHAPE,
	TEXT=0x80,
	BITMAP=0x90
};

enum{				//edit-flag
	UNABLE=0x00,
	MOVEONLY=0x01,
	EDITABLE=0x03,
};

enum{				//attr-flag
	FGCOL=0x01,
	BGCOL=0x02,
	FGPAT=0x04,
	BGPAT=0x08,
	FONT=0x10,
	FNTSZ=0x20,
	PENSZ=0x40
};

typedef struct{				//for input
	uint16	flags;
	float	x,y,tilt_x,tilt_y,pressure;
	uint32	eraser,buttons,modifiers,transit;
	uint64	when;
	BPoint	where;
}input;

typedef struct{
	font_family	fam;
	font_style	sty;
}font;

typedef struct{
	uchar	f;				//0.frame-off  1.frame-on
	BRect	rect;
}s_sel;

typedef struct{
//	char	mode;			//draw mode
//							//-1..to external  0..pick  1..rect  2..ellipse  3..text  4..bitmap  5..line
	int		fc,bc;			//pattern-color
	int		fg,bg;			//pattern no.
	int		fon;			//font no.
	char	fos;			//font size
	char	pen;			//pen no.(size)
//		uchar	sty;			//style(joint,corner)
}s_attr;

typedef struct{
	char	sf;				//select  -1..inner(move)  0..not selected  1-4..corner(edit)
	uchar	type;			//1..rect  2..ellipse
	void	*dp;			//pointer
}data;

typedef struct{
	BRect	rec;			//bounds
	uchar	fc,bc;			//color no.
	uchar	fg,bg;			//pattern no.
	uchar	pen;			//pen no.
	uint	fon;			//font no.
	uchar	fos;			//font size
	uchar	ptc;			//count of points
	char	*str;			//text
	BBitmap	*bm;			//bitmap
}d_all;
//========================================================
class object
{
	public:
		object();
		object(const BPoint &p,const s_attr &m,int points=1/*,ulong support=0*/);
		virtual	~object();
//		virtual object&	operator=(const object &org);
		virtual object*	copy();
		virtual int		create(const uchar &mf,const BPoint &mp);	//mf=1 down,mf=2 up,mf=3 moved
		virtual int		select_chk(const BPoint &p);
		virtual int		select_chk(const BRect &r);
		virtual void	set_attr(const s_attr &m,const uint32 &f=-1);
		virtual	void	move(const BPoint &d);
		virtual void	inspoint(const BPoint &d);
		virtual bool	delpoint();
		virtual void	draw(BBitmap *bm);
		virtual void	drawframe(BView *v,float scale);
		virtual void	form2ctrlp();
		virtual void	ctrlp2form();
		virtual void	read(BFile &f);
		virtual void	write(BFile &f);
		virtual void	settext(const char *s,const uchar &f);
		virtual void	setbitmap(const BRect &r,const uint32 *d);
		virtual int		draw2bm(BBitmap &bm);
		void	chk_usedef();
		void	drawpoint(BView *v,float scale);
		void	overlap(uint32 *p,uint32 c);
		void	obj2ras();
		uchar	objtype,editflag,attrflag;
		int		sel_pt;		//-1.inner  0.notselect  1-.point(corner)
		int		cpts;		//created points	-1.complete
		int		pts;
		BPoint	*pt;
		BRect	rbounds;	//bounds(raster)
		int16	*wr;		//raster
		d_all	da;
};

class object_r:public object
{
	public:
		object_r();
		object_r(const BPoint &p,const s_attr &m);
		int		create(const uchar &mf,const BPoint &mp);
		int		select_chk(const BPoint &p);
		int		select_chk(const BRect &r);
		void	move(const BPoint &d);
		virtual void	drawframe(BView *v,float scale);
};

class object_l:public object
{
	public:
		object_l();
		object_l(const BPoint &p,const s_attr &m);
		int		create(const uchar &mf,const BPoint &mp);
		int		select_chk(const BPoint &p);
		int		select_chk(const BRect &r);
		void	move(const BPoint &d);
//		void	addpoint(const BPoint &d);
		bool	addf;
};

class obj_rect:public object_r
{
	public:
		obj_rect();
		obj_rect(const BPoint &p,const s_attr &m);
		object*	copy();
		void	set_attr(const s_attr &m,const uint32 &f=-1);
		void	form2ctrlp();
		void	ctrlp2form();
		void	read(BFile &f);
		void	write(BFile &f);
		int		draw2bm(BBitmap &bm);
};

class obj_ellipse:public object_r
{
	public:
		obj_ellipse();
		obj_ellipse(const BPoint &p,const s_attr &m);
		object*	copy();
		void	set_attr(const s_attr &m,const uint32 &f=-1);
		void	drawframe(BView *v,float scale);
		void	form2ctrlp();
		void	ctrlp2form();
		void	read(BFile &f);
		void	write(BFile &f);
		int		draw2bm(BBitmap &bm);
};

class obj_text:public object_r
{
	public:
		obj_text();
		obj_text(const BPoint &p,const s_attr &m);
		~obj_text();
		object*	copy();
		void	set_attr(const s_attr &m,const uint32 &f=-1);
		void	form2ctrlp();
		void	ctrlp2form();
		void	read(BFile &f);
		void	write(BFile &f);
		void	settext(const char *s,const uchar &f);
		int		draw2bm(BBitmap &bm);
};

class obj_bitmap:public object_r
{
	public:
		obj_bitmap();
		obj_bitmap(const BPoint &p,const s_attr &m);
		~obj_bitmap();
		object*	copy();
		void	draw(BBitmap *bm);
		void	form2ctrlp();
		void	ctrlp2form();
		void	read(BFile &f);
		void	write(BFile &f);
		void	setbitmap(const BRect &rec,const uint32 *d);
		int		draw2bm(BBitmap &bm);
};

class obj_line:public object_l
{
	public:
		obj_line();
		obj_line(const BPoint &p,const s_attr &m);
		object*	copy();
		void	set_attr(const s_attr &m,const uint32 &f=-1);
		void	inspoint(const BPoint &d);
		bool	delpoint();
		void	drawframe(BView *v,float scale);
		void	form2ctrlp();
		void	ctrlp2form();
		void	read(BFile &f);
		void	write(BFile &f);
		int		draw2bm(BBitmap &bm);
};

class obj_polygon:public object_l
{
	public:
		obj_polygon();
		obj_polygon(const BPoint &p,const s_attr &m);
		object*	copy();
		void	set_attr(const s_attr &m,const uint32 &f=-1);
		void	inspoint(const BPoint &d);
		bool	delpoint();
		void	drawframe(BView *v,float scale);
		void	form2ctrlp();
		void	ctrlp2form();
		void	read(BFile &f);
		void	write(BFile &f);
		int		draw2bm(BBitmap &bm);
};

class obj_curve:public object_l
{
	public:
		obj_curve();
		obj_curve(const BPoint &p,const s_attr &m);
		object*	copy();
		void	set_attr(const s_attr &m,const uint32 &f=-1);
		void	inspoint(const BPoint &d);
		bool	delpoint();
		void	drawframe(BView *v,float scale);
		void	form2ctrlp();
		void	ctrlp2form();
		void	read(BFile &f);
		void	write(BFile &f);
		int		draw2bm(BBitmap &bm);
};

class obj_shape:public object_l
{
	public:
		obj_shape();
		obj_shape(const BPoint &p,const s_attr &m);
		object*	copy();
		void	set_attr(const s_attr &m,const uint32 &f=-1);
		void	inspoint(const BPoint &d);
		bool	delpoint();
		void	drawframe(BView *v,float scale);
		void	form2ctrlp();
		void	ctrlp2form();
		void	read(BFile &f);
		void	write(BFile &f);
		int		draw2bm(BBitmap &bm);
};
//========================================================
class toolview:public BView
{
	public:
		toolview(BRect frame);
		~toolview();
		void	AttachedToWindow();
		void	Draw(BRect rect);
		void	MouseDown(BPoint pt);
		void	MessageReceived(BMessage *msg);
		void	redraw();
	private:
		BBitmap	*bm;
		char	mode;
};

class selectview:public BView
{
	public:
		selectview(BRect frame);
		~selectview();
		void	AttachedToWindow();
		void	Draw(BRect rect);
		void	MouseDown(BPoint pt);
		void	MessageReceived(BMessage *msg);
		void	redraw();
		void	set(s_attr s);
	private:
		BBitmap	*bm;
		s_attr	now;
		uchar	sf;	//b0..fc/bc    b1..fg/bg
};

class gprodview:public BView
{
	public:
		gprodview(BRect frame);
		~gprodview();
		void	AttachedToWindow();
		void	Draw(BRect rect);
		void	KeyDown(const char *bytes,int32 numBytes);
		void	KeyUp(const char *bytes,int32 numBytes);
		void	MouseDown(BPoint pt);
		void	MouseUp(BPoint pt);
		void	MouseMoved(BPoint pt, uint32 code,const BMessage *msg);
		void	MessageReceived(BMessage *msg);
		void	chkinput();
		void	ptr_move(BPoint pt,uchar bt,input inp);
		void	btn_up(BPoint pt,uchar bt,input inp);
		void	btn_down(BPoint pt,uchar bt,input inp);
		BMessage*	ddmessage();		//for drag&drop
		void	ddcopy(BMessage *msg);	//for drag&drop
		void	select_trans(const char *mime,translator_id *tid,uint32 *fmt);
		void	chgscale();
		void	init();
		void	chkattr();
		void	send2toolv();
		void	send2selv();
		void	redraw(char f=7);
		void	drawframe(BRect r);
	private:
		BBitmap	*bm,*bmdd;	//canvas,drag&drop
		s_sel	sel_old,sel_now;
		char	now_mode;
		s_attr	edat,puat;
		bool	mdf;		//mouse_button  t..down  f..up
		BPoint	mbp;		//mouse_before_point
		BPoint	mmp;		//mouse_move_point
		char	kin;		//keyin
		bool	mcf;		//copyable flag(for mouse-move)
		int		selm;		//select mode(2.move,3.edit,4.delete)
		int		selc;		//count of selected object
		float	scale;
};

class gprodwindow:public BWindow
{
	public:
		gprodwindow(BRect frame);
		bool	QuitRequested();
		void	Zoom(BPoint rec_position,float rec_width,float rec_height);
		void	FrameResized(float width,float height);
		void	DispatchMessage(BMessage *message, BHandler *handler);
		void	setupfonts();
		void	setupmenus();
		void	markmenu(const char *name,const char *item);
		void	get_translator_for_output();
		BMenuBar	*mb;
		BMenu	*mf;	//font menu
		input	inp;
		gprodview	*v;
		BScrollBar	*sbv,*sbh;
		selectview	*sv;
		toolview	*tv;
};

class gprod:public BApplication
{
	public:
		gprod();
		void	RefsReceived(BMessage *msg);
		void	MessageReceived(BMessage *msg);
		void	AboutRequested();
		void	load(entry_ref &eref);
		void	save(entry_ref &dir,const char *name);
		gprodwindow	*w;
};
//========================================================
static BFilePanel	*fp;	//panel for load/save

BRect	canvas,vcanvas;		//canvas size,view size
BTranslatorRoster	*tr;
int32				trc=0;
BList				xlt;

struct xlator{
	translator_id		id;
	uint32				type;
	char				name[251];
	char				mime[251];
};

bool	canvasok;
static object*	obj[MAX_OBJS];
static int		objs;

int		fontc;			//count of fonts
font	*ft;			//font(family,style)
int		dfn;			//default font no.

int		udfc;
uchar	*udf;	//use-def-flag[max(COLORS,PATTERNS,FONTS)](b0.color  b1.pattern  b7.font)
uchar	cx[64];			//xfer color-no.(when load/save)
uchar	px[16];			//xfer pattern-no.(when load/save)
int		*fx;			//xfer font-no.(when load/save)

static const uint32	pal[64]={
	0xff000000,0xff0000ff,0xffff0000,0xffff00ff,0xff00ff00,0xff00ffff,0xffffff00,0xffffffff,
	0xff808080,0xff0000c0,0xffc00000,0xffc000c0,0xff00c000,0xff00c0c0,0xffc0c000,0xffc0c0c0,
	0xffff00c0,0xff00ffc0,0xffffffc0,0xffc000ff,0xffc0ff00,0xffc0ffff,0xff00c0ff,0xffffc000,
	0xffffc0ff,0xffc0ffc0,0xffffc0c0,0xffc0c0ff,0xffc00080,0xff00c080,0xffc0c080,0xff8000c0,
	0xff80c000,0xff80c0c0,0xff0080c0,0xffc08000,0xffc080c0,0xff80c080,0xffc08080,0xff8080c0,
	0x80000000,0x800000ff,0x80ff0000,0x80ff00ff,0x8000ff00,0x8000ffff,0x80ffff00,0x80ffffff,
	0xe0ffffff,0xc0ffffff,0xa0ffffff,0x80ffffff,0x60ffffff,0x40ffffff,0x20ffffff,0x00ffffff,
	0xe0000000,0xc0000000,0xa0000000,0x80000000,0x60000000,0x40000000,0x20000000,0x00000000
};

static const uint16	pat[16][16]={
	{	//through
		0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
		0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000
	},{	//full
		0xffff,0xffff,0xffff,0xffff,0xffff,0xffff,0xffff,0xffff,
		0xffff,0xffff,0xffff,0xffff,0xffff,0xffff,0xffff,0xffff
	},{	//mesh
		0xaaaa,0x5555,0xaaaa,0x5555,0xaaaa,0x5555,0xaaaa,0x5555,
		0xaaaa,0x5555,0xaaaa,0x5555,0xaaaa,0x5555,0xaaaa,0x5555
	},{	//checker(small)
		0x5555,0xffff,0x5555,0xffff,0x5555,0xffff,0x5555,0xffff,
		0x5555,0xffff,0x5555,0xffff,0x5555,0xffff,0x5555,0xffff
	},{	//cross(small)
		0x9999,0x6666,0x6666,0x9999,0x9999,0x6666,0x6666,0x9999,
		0x9999,0x6666,0x6666,0x9999,0x9999,0x6666,0x6666,0x9999
	},{	//checker(large)
		0x4444,0xffff,0x4444,0x4444,0x4444,0xffff,0x4444,0x4444,
		0x4444,0xffff,0x4444,0x4444,0x4444,0xffff,0x4444,0x4444
	},{	//cross(large)
		0x8181,0x4242,0x2424,0x1818,0x1818,0x2424,0x4242,0x8181,
		0x8181,0x4242,0x2424,0x1818,0x1818,0x2424,0x4242,0x8181
	},{	//slush
		0x1111,0x2222,0x4444,0x8888,0x1111,0x2222,0x4444,0x8888,
		0x1111,0x2222,0x4444,0x8888,0x1111,0x2222,0x4444,0x8888
	},{	//back-slush
		0x8888,0x4444,0x2222,0x1111,0x8888,0x4444,0x2222,0x1111,
		0x8888,0x4444,0x2222,0x1111,0x8888,0x4444,0x2222,0x1111
	},{	//circle
		0x6060,0xf0f0,0xf0f0,0x6060,0x0606,0x0f0f,0x0f0f,0x0606,
		0x6060,0xf0f0,0xf0f0,0x6060,0x0606,0x0f0f,0x0f0f,0x0606
	},{	//renga
		0xfefe,0xfefe,0xfefe,0x0000,0xefef,0xefef,0xefef,0x0000,
		0xfefe,0xfefe,0xfefe,0x0000,0xefef,0xefef,0xefef,0x0000
	},{	//takeami
		0x8181,0x1818,0xdbdb,0xdbdb,0x1818,0x8181,0xbdbd,0xbdbd,
		0x8181,0x1818,0xdbdb,0xdbdb,0x1818,0x8181,0xbdbd,0xbdbd
	},{	//chuka
		0xfdfe,0x8502,0xb57a,0xa54a,0xbd5a,0x8142,0xff7e,0x0000,
		0x7eff,0x4281,0x5abd,0x4aa5,0x7ab5,0x0285,0xfefd,0x0000
	},{	//double dia
		0x1000,0x2810,0x4438,0x827c,0x4438,0x2810,0x1000,0x0000,
		0x0010,0x1028,0x3844,0x7c82,0x3844,0x1028,0x0010,0x0000
	},{	//heart
		0x6300,0xf780,0xff80,0xff80,0x7f00,0x3e00,0x1c00,0x0800,
		0x00c6,0x01ef,0x01ff,0x01ff,0x00fe,0x007c,0x0038,0x0010
	},{	//star
		0x0000,0x0200,0x0200,0x0700,0x0700,0x0f80,0xfff8,0x7ff0,
		0x3fe0,0x1fc0,0x1fc0,0x3de0,0x38e0,0x7070,0x6030,0x0000
	}
};

static const uint16 iconpat[8][16]={
	{	//pick
		0x0000,0x0600,0x0600,0x0600,0x0600,0x0600,0x06db,0xc6db,
		0x67ff,0x37fe,0x3ffe,0x1ffc,0x0ff8,0x07f8,0x07f8,0x07f8
	},{	//rect
		0x0000,0x0000,0x0000,0x0000,0x3ffc,0x2004,0x2004,0x2004,
		0x2004,0x2004,0x2004,0x3ffc,0x0000,0x0000,0x0000,0x0000
	},{	//ellipse
		0x0000,0x0000,0x0000,0x0000,0x03c0,0x0c30,0x1008,0x2004,
		0x2004,0x1008,0x0c30,0x03c0,0x0000,0x0000,0x0000,0x0000
	},{	//line
		0x0000,0x0000,0x0004,0x000c,0x0018,0x0028,0x0050,0x0090,
		0x0120,0x0220,0x0440,0x0840,0x1080,0x2080,0x0000,0x0000
	},{	//polygon
		0x0000,0x0000,0x0004,0x000c,0x0018,0x0028,0x0050,0x0090,
		0x0120,0x0220,0x0440,0x0840,0x1080,0x3f80,0x0000,0x0000
	},{	//curve
		0x0000,0x0000,0x0000,0x0030,0x0048,0x0088,0x0110,0x0210,
		0x0420,0x0820,0x0840,0x1040,0x1080,0x1080,0x0000,0x0000
	},{	//shape
		0x0000,0x0000,0x0000,0x0030,0x0048,0x0088,0x0110,0x0210,
		0x0420,0x0820,0x0840,0x1040,0x1080,0x0f00,0x0000,0x0000
	},{	//to ext
		0x0000,0x2000,0x0000,0x27ce,0x254a,0x254e,0x0002,0x0f8e,
		0x0f80,0x0780,0x0f80,0x1d80,0x3800,0x3000,0x0000,0x0000
	}
};

static const uchar	triangle[4][8]={
	{	//up
		0x00,0x00,0x18,0x3c,0x7e,0xff,0x00,0x00
	},{	//down
		0x00,0x00,0xff,0x7e,0x3c,0x18,0x00,0x00
	},{	//left
		0x20,0x30,0x38,0x3c,0x3c,0x38,0x30,0x20
	},{	//right
		0x04,0x0c,0x1c,0x3c,0x3c,0x1c,0x0c,0x04
	}
};
