// gprop.h		ver.0.04
//copyright 2004 Beer[Tsutsumi Hiroyuki]

#include	<Be.h>
#if	B_BEOS_VERSION>=0x0520
#include	<Locale.h>
typedef	const char	Char;
#else
#define	_T(str)	str
typedef	char		Char;
#endif

#define	SELECTVIEWCOLOR	0x00ffcc44
#define	TOOLVIEWCOLOR	0x00ffcc44

//for input(tablet,mouse,key)
enum{				//flags
	TABLET_XY=		0x0001,		//be:tablet_x,be:tablet_y
	TABLET_TILT_XY=	0x0002,		//be:tablet_tilt_x,be:tablet_tilt_y
	TABLET_PRESSURE=0x0004,		//be:tablet_pressure
	TABLET_ERASER=	0x0008,		//be:tablet_eraser
	BUTTONS=		0x0010,		//buttons
	MODIFIERS=		0x0020,		//modifiers
	WHEN=			0x0100,		//when
	WHERE=			0x0200,		//be:view_where
	TRANSIT=		0x0400,		//be:transit
};

enum{				//mouse
	NONE=0,
	PRESS,
	RELEASE,
	PRESSMOVE,
	RELEASEMOVE,
	CANSEL
};

enum{				//object-type
//	PICK=0,
	DOT=0x01,		STAMP=0x02,
	FREE=0x10,		SAND=0x11,			SPRAY=0x12,
	LINE=0x20,		CURVE=0x21,
	POLY=0x40,		FILLPOLY=0x41,
	RECT=0x50,		FILLRECT=0x51,
	ELLIPSE=0x60,	FILLELLIPSE=0x61,
	TEXT=0x80,
	BITMAP=0x90,
	PAINT=0xe0,
	EXT=0xff
};

typedef struct{				//for input
	uint16	flags;
	float	x,y,tilt_x,tilt_y,pressure;
	uint32	eraser,buttons,modifiers,transit;
	uint64	when;
	BPoint	where;
}input;

typedef struct{
	font_family	fam;
	font_style	sty;
}font;

typedef struct{
	uchar	f;				//0.frame-off  1.frame-on
	BRect	rect;
}s_sel;

typedef struct{
	uchar	mode;			//draw mode
	uint32	fc,bc;			//color
	uchar	pen;			//pen no.(size)
	uchar	fos;			//font size
	font	fon;			//font name
	char	stp[256];		//stamp name
}s_mode;

typedef struct{
	char	sf;				//select  -1..inner(move)  0..not selected  1-4..corner(edit)
	uchar	type;			//1..rect  2..ellipse
	void	*dp;			//pointer
}data;

typedef struct{
	uint32	fc,bc;			//color
	BBitmap	*bm;
}d_paint;

typedef struct{
	BRect	rec;
	uint32	fc,bc;			//color
	uchar	pen;			//pen no.
}d_rect;

typedef struct{
	BRect	rec;
	uint32	fc,bc;			//color
	uchar	pen;			//pen no.
}d_ellipse;

typedef struct{
	BRect	rec;
	uint32	fc,bc;			//color
	uchar	fos;			//font size
	font	fon;			//font name
	char	*str;			//text
}d_text;

typedef struct{
	BRect	rec;
	BBitmap	*bm;
}d_bitmap;

typedef struct{
	uchar	ptc;
//	BPoint	*pt;
	uint32	fc,bc;			//color
	uchar	pen;			//pen no.(size)
}d_line;

typedef struct{
	uchar	ptc;
//	BPoint	*pt;
	uint32	fc,bc;			//color
	uchar	pen;			//pen no.(size)
}d_poly;
//========================================================
class object
{
	public:
		object();
		object(const BPoint &p,const s_mode &m,int points=1);
		virtual	~object();
		virtual int		create(const uchar &mf,const BPoint &mp);	//mf=1 down,mf=2 up,mf=3 moved
		virtual void	set_attr(const s_mode &m,const uint32 &f=-1);
		virtual void	draw(BBitmap *bm);
		virtual void	drawframe(BView *v,float scale);
		virtual void	ctrlp2form();
		virtual void	settext(const char *s,const uchar &f);
		virtual void	setbitmap(const BRect &r,const uint32 *d);
		int		pts;
		BPoint	*pt;
		int		cpts;		//created points
		BRect	bounds;
};

class object_r:public object
{
	public:
		object_r();
		object_r(const BPoint &p,const s_mode &m);
		int		create(const uchar&,const BPoint&);
};

class object_l:public object
{
	public:
		object_l();
		object_l(const BPoint &p,const s_mode &m);
		int		create(const uchar&,const BPoint&);
};

class obj_paint:public object
{
	public:
		obj_paint(const BPoint &p,const s_mode &m,const BBitmap *bm);
		~obj_paint();
		int		create(const uchar&,const BPoint&);
		void	set_attr(const s_mode &m,const uint32 &f=-1);
		void	draw(BBitmap *bm);
//		void	drawframe(BView *v,float scale);
		d_paint	dp;
};

class obj_rect:public object_r
{
	public:
		obj_rect();
		obj_rect(const BPoint &p,const s_mode &m);
		void	set_attr(const s_mode &m,const uint32 &f=-1);
		void	draw(BBitmap *bm);
		void	drawframe(BView *v,float scale);
		void	ctrlp2form();
		d_rect	dr;
};

class obj_ellipse:public object_r
{
	public:
		obj_ellipse();
		obj_ellipse(const BPoint &p,const s_mode &m);
		void	set_attr(const s_mode &m,const uint32 &f=-1);
		void	draw(BBitmap *bm);
		void	drawframe(BView *v,float scale);
		void	ctrlp2form();
		d_ellipse	de;
};

class obj_text:public object_r
{
	public:
		obj_text();
		obj_text(const BPoint &p,const s_mode &m);
		~obj_text();
		void	set_attr(const s_mode &m,const uint32 &f=-1);
		void	draw(BBitmap *bm);
//		void	drawframe(BView *v,float scale);
		void	ctrlp2form();
		void	settext(const char *s,const uchar &f);
		d_text	dt;
};

class obj_bitmap:public object_r
{
	public:
		obj_bitmap();
		obj_bitmap(const BPoint &p,const s_mode &m);
		~obj_bitmap();
		void	draw(BBitmap *bm);
//		void	drawframe(BView *v,float scale);
		void	ctrlp2form();
		void	setbitmap(const BRect &rec,const uint32 *d);
		d_bitmap	db;
};

class obj_line:public object_l
{
	public:
		obj_line();
		obj_line(const BPoint &p,const s_mode &m);
		void	set_attr(const s_mode &m,const uint32 &f=-1);
		void	draw(BBitmap *bm);
		void	drawframe(BView *v,float scale);
		d_line	dl;
};

class obj_polygon:public object_l
{
	public:
		obj_polygon();
		obj_polygon(const BPoint &p,const s_mode &m);
		void	set_attr(const s_mode &m,const uint32 &f=-1);
		void	draw(BBitmap *bm);
		void	drawframe(BView *v,float scale);
		d_poly	dp;
};
//========================================================
class toolview:public BView
{
	public:
		toolview(BRect frame);
		~toolview();
		void	AttachedToWindow();
		void	Draw(BRect rect);
		void	MouseDown(BPoint pt);
		void	MessageReceived(BMessage *msg);
		void	redraw();
	private:
		BBitmap	*bm;
		uchar	mode;
};

class selectview:public BView
{
	public:
		selectview(BRect frame);
		~selectview();
		void	AttachedToWindow();
		void	Draw(BRect rect);
		void	MouseDown(BPoint pt);
		void	MessageReceived(BMessage *msg);
		void	redraw();
		void	set(s_mode s);
	private:
		BBitmap	*bm;
		s_mode	now;
		uchar	sf;	//b0..fc/bc
};

class gpropview:public BView
{
	public:
		gpropview(BRect frame);
		~gpropview();
		void	AttachedToWindow();
		void	Draw(BRect rect);
		void	Pulse();
		void	MouseDown(BPoint pt);
		void	MouseUp(BPoint pt);
		void	MouseMoved(BPoint pt,uint32 code,const BMessage *msg);
		void	MessageReceived(BMessage *msg);
		void	chkinput();
		void	ptr_move(BPoint pt,uchar bt,input inp);
		void	btn_up(BPoint pt,uchar bt,input inp);
		void	btn_down(BPoint pt,uchar bt,input inp);
		BMessage*	ddmessage();		//for drag&drop
		void	ddcopy(BMessage *msg);	//for drag&drop
		void	select_trans(const char *mime,translator_id *tid,uint32 *fmt);
		void	newcanvas(BRect r,BBitmap *dbm=NULL);
		void	chgscale();
		void	init();
		void	send2toolv();
		void	send2selv();
		void	setstpbmp();
		void	setstpcol();
		void	redraw(char f=0xff);
		void	drawframe(BRect r);
		void	load(entry_ref &eref);
		void	save(entry_ref &dir,char *name);
	private:
		BView	*vv;		//for draw bitmap
		BBitmap	*bm,*bmm,*bmdd,*stbm[4];	//canvas,memory(undo),drag&drop,stamp
		BFilePanel	*fp;	//panel for load/save
		int16		xfno;
		char	title[256],mime[256];
		s_sel	sel_old,sel_now;
		s_mode	now;
		bool	mdf;		//mouse_button  t..down  f..up
		BPoint	mbp;		//mouse_before_point
		BPoint	mmp;		//mouse_move_point
		uint32	mbt;		//mouse_before_time
		int		bps;		//before_pensize
		bool	mcf;		//copyable flag(for mouse-move)
		int		selm;		//select mode
		float	scale;
};

class gpropwindow:public BWindow
{
	public:
		gpropwindow(BRect frame);
		bool	QuitRequested();
		void	Zoom(BPoint rec_position,float rec_width,float rec_height);
		void	FrameResized(float width,float height);
		void	DispatchMessage(BMessage *message, BHandler *handler);
		BMenu*	setupfonts();
		void	setupmenus();
		void	markmenu(const char *name,const char *item);
		void	get_translator_for_output();
		BMenuBar	*mb;
		input	inp;
		gpropview	*v;
		BScrollBar	*sbv,*sbh;
		selectview	*sv;
		toolview	*tv;
};

class gprop:public BApplication
{
	public:
		gprop();
		void	RefsReceived(BMessage *mes);
		void	MessageReceived(BMessage *mes);
		void	AboutRequested();
		gpropwindow	*w;
};
//========================================================
BRect	canvas,vcanvas;		//canvas size,view size
BTranslatorRoster	*tr;
int32				trc=0;
BList				xlt;

struct xlator{
	translator_id		id;
	uint32				type;
	char				name[251];
	char				mime[251];
};

//int	screen_width,screen_height;

bool	initok;
static object*	obj;
BPath	stpdir;

static const uint32	pal[64]={
	0xff000000,0xff0000ff,0xffff0000,0xffff00ff,0xff00ff00,0xff00ffff,0xffffff00,0xffffffff,
	0xff008000,0xff0080ff,0xffff8000,0xffff80ff,0xff00c000,0xff00c0ff,0xffffc000,0xffffc0ff,
	0xff000080,0xff0000c0,0xffff0080,0xffff00c0,0xff00ff80,0xff00ffc0,0xffffff80,0xffffffc0,
	0xff008080,0xff0080c0,0xffff8080,0xffff80c0,0xff00c080,0xff00c0c0,0xffffc080,0xffffc0c0,
	0xff800000,0xff8000ff,0xffc00000,0xffc000ff,0xff80ff00,0xff80ffff,0xffc0ff00,0xffc0ffff,
	0xff808000,0xff8080ff,0xffc08000,0xffc080ff,0xff80c000,0xff80c0ff,0xffc0c000,0xffc0c0ff,
	0xff800080,0xff8000c0,0xffc00080,0xffc000c0,0xff80ff80,0xff80ffc0,0xffc0ff80,0xffc0ffc0,
	0xff808080,0xff8080c0,0xffc08080,0xffc080c0,0xff80c080,0xff80c0c0,0xffc0c080,0xffc0c0c0,
};

static const uint16 iconpat[10][16]={
	{	//free
		0x0000,0x0000,0x000c,0x0010,0x0020,0x0040,0x0040,0x0080,
		0x0080,0x0100,0x0380,0x0e40,0x1240,0x2180,0x0000,0x0000
	},{	//sand
		0x0000,0x06c0,0x1b60,0x03b0,0x6ad8,0x37ac,0x6eec,0x37bb,
		0xcfec,0x37b0,0x1ab6,0x66ec,0x3580,0x0360,0x06c0,0x0000,
	},{	//spray
		0x0000,0x0280,0x0b60,0x03a0,0x2ad8,0x17a8,0x6eec,0x36ba,
		0x5aec,0x37b0,0x1ab4,0x26e8,0x1580,0x0360,0x0280,0x0000,
	},{	//rect
		0x0000,0x0000,0x0000,0x0000,0x3ffc,0x2004,0x2004,0x2004,
		0x2004,0x2004,0x2004,0x3ffc,0x0000,0x0000,0x0000,0x0000
	},{	//ellipse
		0x0000,0x0000,0x0000,0x0000,0x03c0,0x0c30,0x1008,0x2004,
		0x2004,0x1008,0x0c30,0x03c0,0x0000,0x0000,0x0000,0x0000
	},{	//line
		0x0000,0x0000,0x0004,0x000c,0x0018,0x0028,0x0050,0x0090,
		0x0120,0x0220,0x0440,0x0840,0x1080,0x2080,0x0000,0x0000
	},{	//polygon
		0x0000,0x0000,0x0004,0x000c,0x0018,0x0028,0x0050,0x0090,
		0x0120,0x0220,0x0440,0x0840,0x1080,0x3f80,0x0000,0x0000
	},{	//paint
		0x0000,0x0180,0x0180,0x03c0,0x03c0,0x07e0,0x07e0,0x03c0,
		0x0000,0x0180,0x0180,0x03c0,0x03c0,0x07e0,0x07e0,0x03c0,
	},{	//stamp
		0x0180,0x03c0,0x33cc,0x799e,0x781e,0x300c,0x03c0,0x0ff0,
		0x0ff0,0x1ff8,0x1ff8,0x1ff8,0x0ff0,0x0ff0,0x03c0,0x0000,
	},{	//to ext
		0x0000,0x2000,0x0000,0x27ce,0x254a,0x254e,0x0002,0x0f8e,
		0x0f80,0x0780,0x0f80,0x1d80,0x3800,0x3000,0x0000,0x0000
	}
};

static const uchar	triangle[4][8]={
	{	//up
		0x00,0x00,0x18,0x3c,0x7e,0xff,0x00,0x00
	},{	//down
		0x00,0x00,0xff,0x7e,0x3c,0x18,0x00,0x00
	},{	//left
		0x20,0x30,0x38,0x3c,0x3c,0x38,0x30,0x20
	},{	//right
		0x04,0x0c,0x1c,0x3c,0x3c,0x1c,0x0c,0x04
	}
};
